// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by
//            pkgmaker 0.1
//            Thursday, December 11 1997
//
// Module: Statistic
// Author: Hassan Siddiqui
//
// Description: Computes simple statistics on-the-fly from a given input data list.
//
// Comments:
//
// Modifications: See ChangeLog
//
// $Id: Statistic.h,v 1.12 2009/09/01 15:50:54 aibarra Exp $


#ifndef STATISTIC_H
#define STATISTIC_H

// General classes
#include <string> // put this up at the top as order matters
#include <iostream>
#include <cmath>
#include <list>

// SAS-specific classes
//#include "DataFilter.h"
#include "RowRange.h"
#include "StatisticElement.h"

// Utility functions
#include "errstr.h"
#include "Tracer.h"
#include "GenericOperation.h"

// ----------------------------------------------------------------

// class Statistic

// Computes simple statistics on-the-fly from a given input data list.

// NOTE : Before using any of the accessors, any code using this class must check whether
// an instance of this has any statistical information by checking the size() method.

// -----------------------------------------------------------------

class Statistic
{

public:

  /* Constructor */
  Statistic();

  Statistic(const Table& table,
	    const string& name,
	    const vector<RowRange>& rowranges);

  Statistic(const Column * col);

  void updateBasicStatistics(const StatisticElement& element);

  void updateDeviation(const StatisticElement& element);


  /* Accessors */
  const StatisticElement& max()  const;
  const StatisticElement& min() const;
  /* AI gcc4 porting
   * error: type qualifiers ignored on function return type
   * => remove const
   */
   double mean() const;
   double sum() const;
   double rootMeanSquared()   const;
   double standardDeviation() const;
   unsigned long size() const;


  /* Output stream method */

  void write(ostream& stream) const;

private:

  string _name;

  unsigned int _size;

  double _sum;

  double _sumOfSquares;

  double _deviation;

  StatisticElement _min;

  StatisticElement _max;

  bool _meanset; // Used to ensure that there is a mean value computed for the deviation calculation. This is a bit messy.

};


/* I/O Stream functions */

ostream &operator<<(ostream& stream, const Statistic& statistic);


#endif

