// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: TableStatistics
// Author: Hassan Siddiqui
//
// Description: Compute statistics for a given table
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: TableStatistics.h,v 1.12 2001/12/18 11:26:54 gvacanti Exp $


#ifndef TABLESTATISTICS_H 
#define TABLESTATISTICS_H

// General classes

#include <iostream>
#include <string>
#include <map>
#include <vector>

// SAS-specific classes

#include "Dal.h"
//#include "DataFilter.h"
#include "RowRange.h"
#include "errstr.h"

// Utility functions
#include "Statistic.h"
#include "Tracer.h"


// ----------------------------------------------------------------
// class TableStatistics

// Computes, for an given table, 
// the basic statistics for the specified rows.

// ----------------------------------------------------------------

class TableStatistics: public map<string, Statistic, less<string> >
{

public:

  // Constructor

  TableStatistics();
 
  TableStatistics(const Table& table, 
		  const vector<string>& columnNames, 
		  const vector<RowRange>& rowranges); 

  TableStatistics(const Table& table, 
		  const vector<string>& columnNames); 
	

  /* Output stream function */
  void write(ostream& stream) const;

private:

  void generateStatistics(const Table& table, 
		     const vector<string>& columnNames, 
		     const vector<RowRange>& rowranges);

};


/* I/O Stream functions */

ostream & operator << (ostream& stream, 
		       const TableStatistics& tablestatistics);

#endif

