// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: testTableStatistics
// Author: Hassan Siddiqui
//
// Description: Test harness for TableStatistics class
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: testTableStatistics.cc,v 1.7 2001/12/18 11:26:54 gvacanti Exp $




// General classes

#include <iostream>


// SAS-specific classes

#include <string>
#include "Dal.h"
#include "FileLocator.h"
#include "DataFilter.h"

// General utility functions
#include <vector>
#include "TableStatistics.h"
#include "Tracer.h"
#include "RowRange.h"

static const unsigned long NUMBER_OF_ROWS = 10;
 
// ---------------------------------------------------

// Main function

// ---------------------------------------------------

int main()
{
  PROC;

  vector<string> namelist;
  string name;

  // Write dataset

  DataSet* set = dataSetServer->open("testTableStatistics.fits", DataSet::Create);

  Table* table = set->addTable("TEST_TABLE", NUMBER_OF_ROWS);

  // Int8 column

  name = "TESTI8_COLUMN";
  namelist.push_back(name);

  Column* testi8Col = table->addColumn(name, Column::Int8, "int8 test data", "");
  ColumnData* testi8ColData = testi8Col->data();
  int8* testi8Data = testi8ColData->int8Data();
  

  // Int16 column

  name = "TESTI16_COLUMN";
  namelist.push_back(name);

  Column* testi16Col = table->addColumn(name, Column::Int16, "int16 test data", "");
  ColumnData* testi16ColData = testi16Col->data();
  int16* testi16Data = testi16ColData->int16Data();
  

  // Real32 column

  name = "TESTR32_COLUMN";
  namelist.push_back(name);

  Column* testr32Col = table->addColumn(name, Column::Real32, "real32 test data", "");
  ColumnData* testr32ColData = testr32Col->data();
  real32* testr32Data = testr32ColData->real32Data();
  
  for (unsigned int i  = 0; i < table->rows(); ++i)
    {
      testi8Data[i] = i;
      testi16Data[i] = i*10;
      testr32Data[i] = i*100;

    }

  // Generate statistics
  RowRange rowrange;
  rowrange.startRow = 0;
  rowrange.stopRow = table->rows()-1;
  vector<RowRange> rowranges;
  rowranges.push_back(rowrange);


  TableStatistics tablestatistics(*table, namelist, rowranges);

  cout << "Table statistics are : " << endl
       << tablestatistics 
       << endl;


  // Do some checks

  assert(tablestatistics["TESTR32_COLUMN"].size() == 10);
  assert(tablestatistics["TESTR32_COLUMN"].sum() == 4500);
  assert(tablestatistics["TESTR32_COLUMN"].max().value() == 900);

  dataSetServer->close(set);


  RETURN 0;
} 
